/****************************************************************************

	madxlib										 (c) 2005 J. A. Robson



Description:

	Source code for creating a Windows DLL that decodes an MP3 to raw
	PCM. It takes an input buffer of MP3 data and converts it to PCM,
	returning the decoded buffer.

	Based off the madlld/madlldlib sources. Madlld comments have been
	preserved and noted.

$Name:  $
$Date: 2005/03/03 19:52:33 $
$Revision: 1.7 $




Licensing:

	[from madlld.c] (c) 2001--2004 Bertrand Petit

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.

	2. Redistributions in binary form must reproduce the above
	copyright notice, this list of conditions and the following
	disclaimer in the documentation and/or other materials provided
	with the distribution.

	3. Neither the name of the author nor the names of its contributors
	may be used to endorse or promote products derived from this
	software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
	TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
	PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
	USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
	OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
	OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
	SUCH DAMAGE.

****************************************************************************/





//
// Includes
//
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>
#include "madxlib.h"






//
// Return an error string associated with a mad error code.
//
// Mad version 0.14.2b introduced the mad_stream_errorstr()
// function. For previous library versions a replacement is
// provided below.
//

#if (MAD_VERSION_MAJOR>=1) || \
    ((MAD_VERSION_MAJOR==0) && \
     (((MAD_VERSION_MINOR==14) && \
       (MAD_VERSION_PATCH>=2)) || \
      (MAD_VERSION_MINOR>14)))
#define MadErrorString(x) mad_stream_errorstr(x)
#else
static const char *
MadErrorString (const struct mad_stream *Stream)
{
  switch (Stream->error)
    {
      // Generic unrecoverable errors.
    case MAD_ERROR_BUFLEN:
      return ("input buffer too small, or EOF");
    case MAD_ERROR_BUFPTR:
      return ("invalid (null) buffer pointer");
    case MAD_ERROR_NOMEM:
      return ("not enough memory");

      // Frame header related unrecoverable errors.
    case MAD_ERROR_LOSTSYNC:
      return ("lost synchronization");
    case MAD_ERROR_BADLAYER:
      return ("reserved header layer value");
    case MAD_ERROR_BADBITRATE:
      return ("forbidden bitrate value");
    case MAD_ERROR_BADSAMPLERATE:
      return ("reserved sample frequency value");
    case MAD_ERROR_BADEMPHASIS:
      return ("reserved emphasis value");

      // Recoverable errors
    case MAD_ERROR_BADCRC:
      return ("CRC check failed");
    case MAD_ERROR_BADBITALLOC:
      return ("forbidden bit allocation value");
    case MAD_ERROR_BADSCALEFACTOR:
      return ("bad scalefactor index");
    case MAD_ERROR_BADFRAMELEN:
      return ("bad frame length");
    case MAD_ERROR_BADBIGVALUES:
      return ("bad big_values count");
    case MAD_ERROR_BADBLOCKTYPE:
      return ("reserved block_type");
    case MAD_ERROR_BADSCFSI:
      return ("bad scalefactor selection info");
    case MAD_ERROR_BADDATAPTR:
      return ("bad main_data_begin pointer");
    case MAD_ERROR_BADPART3LEN:
      return ("bad audio data length");
    case MAD_ERROR_BADHUFFTABLE:
      return ("bad Huffman table select");
    case MAD_ERROR_BADHUFFDATA:
      return ("Huffman data overrun");
    case MAD_ERROR_BADSTEREO:
      return ("incompatible block_type for JS");


      // Unknown error. This switch may be out of
      // sync with libmad's defined error codes.
    default:
      return ("Unknown error code");
    }
}
#endif






//
// Borrowed from audio.c in the madplay source.
// 32-bit pseudo-random number generator.
//

static inline unsigned long
psurand_gen (unsigned long state)
{
  return (state * 0x0019660dL + 0x3c6ef35fL) & 0xffffffffL;
}





//
// scale_audio():
//
// A scaling routine based on madplay's audio_linear_dither()
// function. It performs dithering, which is the addition of
// a random number to the least significant bits (LSB) of
// the sample that targets the LSB at the 16 bit mark.
// madx_read() uses this to perform its scaling before
// output.
//

static inline signed short
scale_audio (mad_fixed_t sample, mad_fixed_t * rndval_ptr)
{
  unsigned int scalebits;
  mad_fixed_t output, mask, rnd;

  // bias
  output = sample + (1L << (MAD_F_FRACBITS + 1 - 16 - 1));

  scalebits = MAD_F_FRACBITS + 1 - 16;
  mask = (1L << scalebits) - 1;

  // dither
  rnd = psurand_gen (*rndval_ptr);
  output += (rnd & mask) - (*rndval_ptr & mask);
  *rndval_ptr = rnd;

  // clip
  if (output >= MAD_F_ONE)
    output = MAD_F_ONE - 1;
  else if (output < -MAD_F_ONE)
    output = -MAD_F_ONE;

  // quantize
  output &= ~mask;

  // scale
  return output >> scalebits;
}









//
// madx_init():
//
// Initializes mxhouse members, open files and
// psuedo-files.
//

extern "C"
__declspec (dllexport)
     int __stdcall madx_init (unsigned char *out_buffer, madx_house * mxhouse)
{



  // Initialize libmad structures
  mad_stream_init (&mxhouse->stream);
  mad_frame_init (&mxhouse->frame);
  mad_synth_init (&mxhouse->synth);
  mad_timer_reset (&mxhouse->timer);


  // Assign pointer to output buffer. "output_ptr"
  // is used to determine when to flush the output
  // buffer in madx_read().

  mxhouse->output_ptr = out_buffer;


  return (1);


}








//
// madx_read():
//
// Is called until EOF is reached or error
// occurs. The calling code must inspect
// the "madx_sig" return values to
// determine what to do (e.g. call
// madx_read() again, flush output, etc).
//

extern "C"
__declspec (dllexport)
     madx_sig __stdcall
       madx_read (unsigned char *in_buffer,
		  unsigned char *out_buffer, madx_house * mxhouse,
		  madx_stat * mxstat)
{


  // "Random" seed for generating dither. Used
  // by scale_audio() in preparing the samples
  // for 16 bit output.

  mad_fixed_t rnscale = 0xa8b9ff7e;

  unsigned char *guard_ptr = NULL;
  unsigned char *readstart;

  // Clear mxstat->msg
  sprintf (mxstat->msg, "");



  mxstat->buffstart = 0;


  // Input file has been read completely
  if (mxstat->is_eof && !mxstat->readsize)
    {

      sprintf (mxstat->msg, "End of input stream");
      return (EOF_REACHED);
    }



  // Executes on initial loop

  if ((mxstat->readsize == 0) &&
      (mxstat->remaining == 0) && mxhouse->stream.buffer == NULL)
    {

      mxstat->readsize = MADX_INPUT_BUFFER_SIZE;
      mxstat->remaining = 0;
      return (MORE_INPUT);
    }


  readstart = in_buffer;




  // Stream buffer

  if (mxhouse->stream.buffer == NULL ||
      mxhouse->stream.error == MAD_ERROR_BUFLEN)
    {


      // BUFFER_GUARD
      // "mxstat->is_eof" must be set to
      // non-zero IMMEDIATELY (see "test.cpp")
      // once it is detected, so the following
      // may execute, padding the buffer.

      if (mxstat->is_eof)
	{

	  guard_ptr = in_buffer + mxstat->readsize;
	  memset (guard_ptr, 0, MAD_BUFFER_GUARD);

	  mxstat->readsize += MAD_BUFFER_GUARD;
	}



      // Pipe the new buffer content to libmad's
      // stream decoder facility.

      mad_stream_buffer (&mxhouse->stream,
			 in_buffer, mxstat->readsize + mxstat->remaining);

      mxhouse->stream.error = (mad_error) 0;

    }







  // [Madlld comment]:
  // Decode the next MPEG frame. The streams is read from the
  // buffer, its constituents are broken down and stored in the
  // mxhouse->frame structure, ready for examination/alteration
  // or PCM synthesis. Decoding options are carried in the Frame
  // structure from the mxhouse->stream structure.
  //
  // Error handling: mad_frame_decode() returns a non zero value
  // when an error occurs. The error condition can be checked in
  // the error member of the mxhouse->stream structure. A mad error is
  // recoverable or fatal, the error status is checked with the
  // MAD_RECOVERABLE macro.
  //
  // {4} When a fatal error is encountered all decoding
  // activities shall be stopped, except when a MAD_ERROR_BUFLEN
  // is signaled. This condition means that the
  // mad_frame_decode() function needs more input to complete
  // its work. One shall refill the buffer and repeat the
  // mad_frame_decode() call. Some bytes may be left unused at
  // the end of the buffer if those bytes forms an incomplete
  // frame. Before refilling, the remaining bytes must be moved
  // to the beginning of the buffer and used for input for the
  // next mad_frame_decode() invocation. (See the comments
  // marked {2} earlier for more details.)
  //
  // Recoverable errors are caused by malformed bit-streams, in
  // this case one can call again mad_frame_decode() in order to
  // skip the faulty part and re-sync to the next frame.



  if (mad_frame_decode (&mxhouse->frame, &mxhouse->stream))
    {

      if (MAD_RECOVERABLE (mxhouse->stream.error))
	{


	  // Do not print a message if the error is
	  // a loss of synchronization or this loss
	  // is due to the end of stream guard bytes.

	  if (mxhouse->stream.error != MAD_ERROR_LOSTSYNC
	      || mxhouse->stream.this_frame != guard_ptr ||
	      // Suppress error if caused by ID3 tag.
	      (mxhouse->stream.this_frame[0] != 'I' &&	// ID3v2
	       mxhouse->stream.this_frame[1] != 'D' && mxhouse->stream.this_frame[2] != '3') || (mxhouse->stream.this_frame[0] != 'T' &&	// ID3v1
												 mxhouse->
												 stream.
												 this_frame
												 [1]
												 !=
												 'A'
												 &&
												 mxhouse->
												 stream.
												 this_frame
												 [2]
												 !=
												 'G'))
	    {
	      sprintf (mxstat->msg, "Recoverable frame level error (%s)",
		       MadErrorString (&mxhouse->stream));

	    }
	  return (CALL_AGAIN);


	}
      else			// Non-recoverable error
	{


	  // MAD_ERROR_BUFLEN means that more input
	  // is needed (it's not recoverable without
	  // interaction from the calling code). The
	  // bytes already in the buffer must be
	  // preserved. "mxstat->buffstart" is used
	  // to point to the end of the preserved
	  // bytes in "in_buffer", then MORE_INPUT
	  // is requested. "mxstat->buffstart" is set
	  // to 0 every time "madx_read()" is
	  // called. The calling code checks this
	  // variable to determine if a full buffer
	  // read or a partial read is necessary.
	  // (Not "0" signifies a partial read.)

	  if (mxhouse->stream.error == MAD_ERROR_BUFLEN)
	    {


	      sprintf (mxstat->msg,
		       "Need more input (%s)",
		       MadErrorString (&mxhouse->stream));

	      mxstat->remaining =
		mxhouse->stream.bufend - mxhouse->stream.next_frame;

	      memmove (in_buffer,
		       mxhouse->stream.next_frame, mxstat->remaining);

	      mxstat->readsize = MADX_INPUT_BUFFER_SIZE - mxstat->remaining;


	      // How far "forward" in_buffer to begin:
	      mxstat->buffstart = in_buffer + mxstat->remaining;


	      return (MORE_INPUT);

	    }
	  else			// Error returned
	    {
	      sprintf (mxstat->msg, "Unrecoverable frame level error (%s).",
		       MadErrorString (&mxhouse->stream));
	      return (ERROR_OCCURED);
	    }
	}
    }







  // [Madlld comment]:
  // Accounting. The computed frame duration is in the frame
  // header structure. It is expressed as a fixed point number
  // whole data type is mad_timer_t. It is different from the
  // samples fixed point format and unlike it, it can't directly
  // be added or subtracted. The timer module provides several
  // functions to operate on such numbers. Be careful there, as
  // some functions of libmad's timer module receive some of
  // their mad_timer_t arguments by value!

  mxhouse->frame_cnt++;
  mad_timer_add (&mxhouse->timer, mxhouse->frame.header.duration);




  // Once decoded the frame is synthesized to
  // PCM samples. No errors are reported by
  // mad_synth_frame();

  mad_synth_frame (&mxhouse->synth, &mxhouse->frame);






  // [Madlld comment]:
  // Synthesized samples must be converted from libmad's fixed
  // point number to the consumer format. Here we use unsigned
  // 16 bit big endian integers on two channels. Integer samples
  // are temporarily stored in a buffer that is flushed when
  // full.
  //
  // Note: Code to computer "big-endian" has been preserved
  // below. It is simply commented out. This source produces
  // 16-bit "little-endian" PCM samples.


  for (int i = 0; i < mxhouse->synth.pcm.length; i++)
    {
      signed short sample;

      // Left channel
      sample = scale_audio (mxhouse->synth.pcm.samples[0][i], &rnscale);


      // Originally big-endian:
      // *(mxhouse->output_ptr++)=sample>>8;
      // *(mxhouse->output_ptr++)=sample&0xff;


      // 16-bit little-endian
      *(mxhouse->output_ptr++) = ((sample >> 0) & 0xff);
      *(mxhouse->output_ptr++) = ((sample >> 8) & 0xff);



      // Right channel.
      // If the decoded stream is monophonic then
      // the right output channel is the same as
      // the left one.

      if (MAD_NCHANNELS (&mxhouse->frame.header) == 2)
	sample = scale_audio (mxhouse->synth.pcm.samples[1][i], &rnscale);


      // Originally big-endian:
      // *(mxhouse->output_ptr++)=sample>>8;
      // *(mxhouse->output_ptr++)=sample&0xff;


      // 16-bit little-endian
      *(mxhouse->output_ptr++) = ((sample >> 0) & 0xff);
      *(mxhouse->output_ptr++) = ((sample >> 8) & 0xff);

    }


  // Tell calling code buffer size
  mxstat->write_size = mxhouse->output_ptr - out_buffer;

#ifdef NPRINTF
/*
{
void dprintf (char *format, ...);
dprintf("--%d--\n",mxstat->write_size);
}
*/
#endif
  // Indicate that buffer is ready to be
  // written.

  if (mxhouse->synth.pcm.length < 1152
      || MADX_OUTPUT_BUFFER_SIZE - mxstat->write_size < 1152)
    {

      // Reset pointer
      mxhouse->output_ptr = out_buffer;
      return (FLUSH_BUFFER);
    }
  else
    return (CALL_AGAIN);	// Otherwise, continue


}




//
// madx_deinit():
//
// Close files, free memory allocated for
// structures.
//
extern "C"
__declspec (dllexport)
     void __stdcall madx_deinit (madx_house * mxhouse)
{

  // Mad is no longer used, the structures
  // that were initialized must now be
  // cleared.

  mad_synth_finish (&mxhouse->synth);
  mad_frame_finish (&mxhouse->frame);
  mad_stream_finish (&mxhouse->stream);

}



#define VERSIONSTRING "OggDec 1.0\n"

static int quiet = 0;
static int bits = 16;
static int endian = 0;
static int raw = 0;
static int sign = 1;
unsigned char headbuf[44];	/* The whole buffer */







#define WRITE_U32(buf, x) *(buf)     = (unsigned char)((x)&0xff);\
                          *((buf)+1) = (unsigned char)(((x)>>8)&0xff);\
                          *((buf)+2) = (unsigned char)(((x)>>16)&0xff);\
                          *((buf)+3) = (unsigned char)(((x)>>24)&0xff);

#define WRITE_U16(buf, x) *(buf)     = (unsigned char)((x)&0xff);\
                          *((buf)+1) = (unsigned char)(((x)>>8)&0xff);

/*
 * Some of this based on ao/src/ao_wav.c
 */
static int
write_prelim_header (FILE * out, int channels, int samplerate)
{

  int knownlength = 0;

  unsigned int size = 0x7fffffff;
  // int channels = 2;
  // int samplerate = 44100;
  int bytespersec = channels * samplerate * bits / 8;
  int align = channels * bits / 8;
  int samplesize = bits;

  if (knownlength)
    size = (unsigned int) knownlength;

  memcpy (headbuf, "RIFF", 4);
  WRITE_U32 (headbuf + 4, size - 8);
  memcpy (headbuf + 8, "WAVE", 4);
  memcpy (headbuf + 12, "fmt ", 4);
  WRITE_U32 (headbuf + 16, 16);
  WRITE_U16 (headbuf + 20, 1);	/* format */
  WRITE_U16 (headbuf + 22, channels);
  WRITE_U32 (headbuf + 24, samplerate);
  WRITE_U32 (headbuf + 28, bytespersec);
  WRITE_U16 (headbuf + 32, align);
  WRITE_U16 (headbuf + 34, samplesize);
  memcpy (headbuf + 36, "data", 4);
  WRITE_U32 (headbuf + 40, size - 44);

  if (fwrite (headbuf, 1, 44, out) != 44)
    {
      printf ("ERROR: Failed to write wav header: %s\n", strerror (errno));
      return 1;
    }

  return 0;
}

static int
rewrite_header (FILE * out, unsigned int written)
{
  unsigned int length = written;

  length += 44;

  WRITE_U32 (headbuf + 4, length - 8);
  WRITE_U32 (headbuf + 40, length - 44);
  if (fseek (out, 0, SEEK_SET) != 0)
    {
      printf ("ERROR: Failed to seek on seekable file: %s\n",
	      strerror (errno));
      return 1;
    }

  if (fwrite (headbuf, 1, 44, out) != 44)
    {
      printf ("ERROR: Failed to write wav header: %s\n", strerror (errno));
      return 1;
    }
  return 0;
}

#include <stdio.h>
#include <string.h>
//#include "madxlib.h"



void
mainold (int argc, char *argv[])
{

  FILE *in_file;
  FILE *out_file;

  int bytes = 0;
  // Output file and output buffer
  unsigned char in_buffer[MADX_INPUT_BUFFER_SIZE + MAD_BUFFER_GUARD];
  unsigned char out_buffer[MADX_OUTPUT_BUFFER_SIZE];

  size_t a;
  size_t in_size = MADX_INPUT_BUFFER_SIZE + MAD_BUFFER_GUARD;


  // Structure necessary to use madxlib
  madx_house mxhouse;


  madx_stat mxstat;

  // Catch signals from madx_read()
  madx_sig mxsig;


//input 46088
//output 9216

//printf("input buffer %d \n output %d\n",MADX_INPUT_BUFFER_SIZE + MAD_BUFFER_GUARD,MADX_OUTPUT_BUFFER_SIZE);

//exit(1);


  // If not the exact number of command
  // line parameters, quit

  if (argc != 2)
    {
      printf ("usage: %s 'somefile.mp3'\n", argv[0]);
      return;
    }




  // Initialize madxlib

  madx_init (out_buffer, &mxhouse);




  // Open files

  if (!(in_file = fopen (argv[1], "rb")))
    {
      printf ("Error with fopen of input file!\n");
      return;
    }


  if (!(out_file = fopen ("test2.wav", "wb")))
    {
      printf ("Error with fopen of file test.pcm!\n");
      return;
    }


  write_prelim_header (out_file, 2, 44100);


  // This loop reads input until EOF, fills the
  // output buffer, writes the output file, and
  // checks for errors. The return values from
  // madx_read() must be inspected after each
  // call to determine the appropriate action.

  do
    {



      // Read input, pass processed data to out_buffer
      mxsig = madx_read (in_buffer, out_buffer, &mxhouse, &mxstat);

      if (strcmp (mxstat.msg, ""))
	{
	  printf ("%s\n", mxstat.msg);

	}



      if (mxsig == ERROR_OCCURED)	// Error
	{

	  printf ("Unrecoverable error %s\n", mxstat.msg);
	  break;

	}
      else if (mxsig == MORE_INPUT)	// Fill buffer
	{


	  printf ("mxstat.readsize %d\n", mxstat.readsize);

	  if (mxstat.buffstart)	// Fill partial buffer
	    {


	      if ((a =
		   fread (mxstat.buffstart, 1, mxstat.readsize,
			  in_file)) == mxstat.readsize)
		{
		  printf ("Filling buffer.\n");
		}
	      else if (feof (in_file))	// EOF must be flagged
		{
		  mxstat.is_eof = 1;
		  mxstat.readsize = a;


		  printf ("Valor de a e readsize %d %d \n", a,
			  mxstat.readsize);

		}
	      else
		printf ("Error! %d\n", ferror (in_file));

	    }
	  else			// Read full buffer
	    {

	      if (fread (in_buffer, 1, mxstat.readsize, in_file) ==
		  mxstat.readsize)
		{
		  printf ("Fill buffer full.\n");
		}
	      else if (feof (in_file))	// EOF must be flagged
		{
		  mxstat.is_eof = 1;
		  printf ("2 Valor de a e readsize %d %d \n", a,
			  mxstat.readsize);
		  mxstat.readsize = a;

		}
	      else
		printf ("Error! %d\n", ferror (in_file));

	    }


	}
      else if (mxsig == FLUSH_BUFFER)	// Output to file
	{


	  if (fwrite (out_buffer, 1, mxstat.write_size, out_file) ==
	      mxstat.write_size)
	    printf ("Writing buffer.\n");
	  else
	    printf ("fwrite() error\n");

	  bytes += mxstat.write_size;

	}
      else if (mxsig == EOF_REACHED)	// Input file EOF
	{


	  printf ("End of file reached\n");

	  if ((a =
	       fwrite (out_buffer, 1, mxstat.write_size,
		       out_file)) != mxstat.write_size)
	    {
	      printf ("Error with final write! out_size:%d, fwrite: %d\n",
		      mxstat.write_size, a);
	    }
	  else
	    printf ("Finished. out_size:%d, fwrite: %d\n",
		    mxstat.write_size, a);

	  bytes += mxstat.write_size;




	  break;

	}



    }
  while (1);

  rewrite_header (out_file, bytes);

  // Clean-up
  madx_deinit (&mxhouse);
  fclose (out_file);

  return;

}




static unsigned char in_buffer[MADX_INPUT_BUFFER_SIZE + MAD_BUFFER_GUARD];
static unsigned char out_buffer[MADX_OUTPUT_BUFFER_SIZE];

static size_t a;
static size_t in_size = MADX_INPUT_BUFFER_SIZE + MAD_BUFFER_GUARD;


  // Structure necessary to use madxlib
static madx_house mxhouse;


static madx_stat mxstat;

  // Catch signals from madx_read()
static madx_sig mxsig;


static int position = 0;

static int laststatus = 0;

static int already = 0;

enum d
{

  READBUFFER = 1,
  READBUFFERBUFFERSTART,
  FLUSH,
};


extern "C"
{


  void indprintff (char *format, ...);

  int decode_mad_MP3 (char *bufmp3, int len, char *out, int outlimit,
		      int *outsize, int *large)
  {

    int a;


    int force0 = 0;

    static int dec = 0;

    if (0 == dec)
      {

	printf ("Initial decoding\n");
	dec++;
      }


    if (already)
      {
	return -1;
      }

    if (READBUFFERBUFFERSTART == laststatus)
      {


	//printf ("Filling buffer %d \n", len);
	memcpy (mxstat.buffstart, bufmp3, len);

	if (mxstat.readsize != len)
	  {

	    mxstat.is_eof = 1;
	    mxstat.readsize = len;

	    //printf ("Last item read %d\n", mxstat.readsize);
	    force0 = 1;
	  }

	laststatus = 0;
      }

    if (READBUFFER == laststatus)
      {
	//printf ("Filling buffer %d \n", len);
	memcpy (in_buffer, bufmp3, len);

	if (mxstat.readsize != len)
	  {

	    mxstat.is_eof = 1;
	    mxstat.readsize = len;
	    //printf ("Last item read %d\n", mxstat.readsize);
	    force0 = 1;
	  }

	laststatus = 0;
      }
    do
      {



	// Read input, pass processed data to out_buffer


      final:

	mxsig = madx_read (in_buffer, out_buffer, &mxhouse, &mxstat);

	if (strcmp (mxstat.msg, ""))
	  {


	    //printf ("%s\n", mxstat.msg);

	    indprintff ("Madlib debug -> %s", mxstat.msg);
	  }



	if (mxsig == ERROR_OCCURED)	// Error
	  {

	    printf ("Unrecoverable error %s\n", mxstat.msg);
	    return -2;

	  }
	else if (mxsig == MORE_INPUT)	// Fill buffer
	  {




	    if (mxstat.buffstart)	// Fill partial buffer
	      {



		laststatus = READBUFFERBUFFERSTART;

		*large = mxstat.readsize;

		if (force0)
		  {

		    mxstat.is_eof = 1;
		    mxstat.readsize = 0;

		    // printf ("Last real item read %d READBUFFERBUFFERSTART\n",
		    //mxstat.readsize);
		    already = 1;
		    goto final;

		  }
		return -1;

	      }
	    else
	      {
		laststatus = READBUFFER;

		*large = mxstat.readsize;

		if (force0)
		  {

		    mxstat.is_eof = 1;
		    mxstat.readsize = 0;

		    //printf ("Last real item read %d READBUFFER\n",
		    // mxstat.readsize);
		    already = 1;
		    goto final;
		  }
		return -1;

	      }


	  }
	else if (mxsig == FLUSH_BUFFER)	// Output to file
	  {

	    // printf ("Flush buffer %d \n", mxstat.write_size);

	    memcpy (out, out_buffer, mxstat.write_size);

	    *outsize = mxstat.write_size;

	    laststatus = FLUSH;
	    *large = mxstat.readsize;
	    return 0;

	  }
	else if (mxsig == EOF_REACHED)	// Input file EOF
	  {
	    printf ("Final buffer %d \n", mxstat.write_size);

	    memcpy (out, out_buffer, mxstat.write_size);

	    *outsize = mxstat.write_size;

	    laststatus = FLUSH;

	    *large = 4096;


	    return 0;



	  }



      }
    while (1);


    *large = mxstat.readsize;
    return 0;
  }
  int Init_mad_MP3 ()
  {
    laststatus = 0;

    already = 0;
    position = 0;



    memset (&mxsig, 0, sizeof (mxsig));
    memset (&mxstat, 0, sizeof (mxstat));
    memset (&mxhouse, 0, sizeof (mxhouse));


    madx_init (out_buffer, &mxhouse);
    return 0;
  }

  int Exit_mad_MP3 ()
  {

    madx_deinit (&mxhouse);
    memset (&mxhouse, 0, sizeof (mxhouse));
    return 0;
  }


}

/*

int main(int argc , char ** argv)
{
int bytes=0;
	char inputfile[]="orinoco.mp3";
	char output[]="saida.wav";

char buffer[46088]={0,};

char bufferout[46088]={0,};
int ret=1;
int len=0;

int saiu=0;

int large=0;

	FILE *in_file=NULL;
	FILE *out_file=NULL;


  if (argc != 2)
    {
      printf ("usage: %s 'somefile.mp3'\n", argv[0]);
      return 1;
    }
	  if (!(in_file = fopen (argv[1], "rb")))
	    {
	      printf ("Error with fopen of input file %s !\n", argv[1]);
	      return 1;
	    }


	  if (!(out_file = fopen ("test3.wav", "wb")))
	    {
	      printf ("Error with fopen of file test.pcm %s!\n" , "test3.wav");
	      return 1;
    }

 Init_mad_MP3 ();

write_prelim_header (out_file, 2, 44100);

while(1)
{

len=fread(buffer,1,large,in_file);

if(0==len && 0!=large)
{
	printf("Saiu %d %d",len,large);
	break;
}

denovo:

ret= decode_mad_MP3 (buffer, len, bufferout, 9216, &saiu, &large);

printf("LArge %d\n",large);

if(-2==ret)
{
	printf("Error inesperado\n");
	break;
	break;
}

if(0==ret)
{

printf("Filling buffer with %d\n",saiu);
	bytes+=fwrite (bufferout,1,saiu,out_file);
	goto denovo;
}

}

rewrite_header (out_file, bytes);

fclose(out_file);


fclose(in_file);
Exit_mad_MP3 ();

return 0;
}

*/
